@@ -1,22 +1,20 @@
-from __future__ import print_function
-from __future__ import absolute_import
-from future.utils import iteritems
-from collections import OrderedDict
 import functools
 import itertools
 import operator
 import warnings
+from collections import OrderedDict
 
 import docrep
-import xarray as xr
 import numpy as np
+import xarray as xr
 
 from . import comodo
-from .duck_array_ops import _pad_array, _apply_boundary_condition, concatenate
+from .duck_array_ops import _apply_boundary_condition, _pad_array, concatenate
 
 try:
     import numba
-    from .transform import linear_interpolation, conservative_interpolation
+
+    from .transform import conservative_interpolation, linear_interpolation
 except ImportError:
     numba = None
 
@@ -101,9 +99,10 @@ class Axis:
             * None:  Do not apply any boundary conditions. Raise an error if
               boundary conditions are required for the operation.
             * 'fill':  Set values outside the array boundary to fill_value
-              (i.e. a Neumann boundary condition.)
+              (i.e. a Dirichlet boundary condition.)
             * 'extend': Set values outside the array to the nearest array
-              value. (i.e. a limited form of Dirichlet boundary condition.)
+              value. (i.e. a limited form of Neumann boundary condition where
+              the difference at the boundary will be zero.)
             * 'extrapolate': Set values by extrapolating linearly from the two
               points nearest to the edge
             This sets the default value. It can be overriden by specifying the
@@ -125,7 +124,7 @@ class Axis:
             )
         self.boundary = boundary
         if fill_value is not None and not isinstance(fill_value, (int, float)):
-            raise ValueError(f"Expected 'fill_value' to be a number.")
+            raise ValueError("Expected 'fill_value' to be a number.")
         self.fill_value = fill_value if fill_value is not None else 0.0
 
         if coords:
@@ -221,7 +220,7 @@ class Axis:
 
     def _coord_desc(self):
         summary = []
-        for name, cname in iteritems(self.coords):
+        for name, cname in self.coords.items():
             coord_info = "  * %-8s %s" % (name, cname)
             if name in self._default_shifts:
                 coord_info += " --> %s" % self._default_shifts[name]
@@ -259,9 +258,9 @@ class Axis:
             * None:  Do not apply any boundary conditions. Raise an error if
               boundary conditions are required for the operation.
             * 'fill':  Set values outside the array boundary to fill_value
-              (i.e. a Neumann boundary condition.)
+              (i.e. a Dirichlet boundary condition.)
             * 'extend': Set values outside the array to the nearest array
-              value. (i.e. a limited form of Dirichlet boundary condition.)
+              value. (i.e. a limited form of Neumann boundary condition.)
         fill_value : float, optional
             The value to use in the boundary condition with `boundary='fill'`.
         vector_partner : dict, optional
@@ -773,6 +772,7 @@ class Axis:
         target_data=None,
         method="linear",
         mask_edges=True,
+        bypass_checks=False,
         suffix="_transformed",
     ):
         """Convert an array of data to new 1D-coordinates.
@@ -834,6 +834,11 @@ class Axis:
         mask_edges : bool, optional
             If activated, `target` values outside the range of `target_data`
             are masked with nan, by default True. Only applies to 'linear' method.
+        bypass_checks : bool, optional
+            Only applies for `method='linear'`.
+            Option to bypass logic to flip data if monotonically decreasing along the axis.
+            This will improve performance if True, but the user needs to ensure that values
+            are increasing alon the axis.
         suffix : str, optional
             Customizable suffix to the name of the output array. This will
             be added to the original name of `da`. Defaults to `_transformed`.
@@ -850,9 +855,9 @@ class Axis:
         target_dim = None
 
         # check optional numba dependency
-        if numba is None or numba.__version__ < "0.49":
+        if numba is None:
             raise ImportError(
-                "The transform functionality of xgcm requires numba>=0.49. Install/update using `conda install numba` or `conda update numba`"
+                "The transform functionality of xgcm requires numba. Install using `conda install numba`."
             )
 
         # raise error if axis is periodic
@@ -861,6 +866,42 @@ class Axis:
                 "`transform` can only be used on axes that are non-periodic. Pass `periodic=False` to `xgcm.Grid`."
             )
 
+        # complain if the target values are not provided as xr.dataarray
+        for var_name, variable, allowed_types in [
+            ("da", da, [xr.DataArray]),
+            ("target", target, [xr.DataArray, np.ndarray]),
+            ("target_data", target_data, [xr.DataArray]),
+        ]:
+            if not (isinstance(variable, tuple(allowed_types)) or variable is None):
+                raise ValueError(
+                    f"`{var_name}` needs to be a {' or '.join([str(a) for a in allowed_types])}. Found {type(variable)}"
+                )
+
+        def _target_data_name_handling(target_data):
+            """Handle target_data input without a name"""
+            if target_data.name is None:
+                warnings.warn(
+                    "Input`target_data` has no name, but we need a name for the transformed dimension. The name `TRANSFORMED_DIMENSION` will be used. To avoid this warning, call `.rename` on `target_data` before calling `transform`."
+                )
+                target_data.name = "TRANSFORMED_DIMENSION"
+
+        def _check_other_dims(target_da):
+            # check if other dimensions (excluding ones associated with the transform axis) are the
+            # same between `da` and `target_data`. If not provide instructions how to work around.
+
+            da_other_dims = set(da.dims) - set(self.coords.values())
+            target_da_other_dims = set(target_da.dims) - set(self.coords.values())
+            if not target_da_other_dims.issubset(da_other_dims):
+                raise ValueError(
+                    f"Found additional dimensions [{target_da_other_dims-da_other_dims}]"
+                    "in `target_data` not found in `da`. This could mean that the target "
+                    "array is not on the same position along other axes."
+                    " If the additional dimensions are associated witha staggered axis, "
+                    "use grid.interp() to move values to other grid position. "
+                    "If additional dimensions are not related to the grid (e.g. climate "
+                    "model ensemble members or similar), use xr.broadcast() before using transform."
+                )
+
         def _parse_target(target, target_dim, target_data_dim, target_data):
             """Parse target values into correct xarray naming and set default naming based on input data"""
             # if target_data is not provided, assume the target to be one of the staggered dataset dimensions.
@@ -879,29 +920,20 @@ class Axis:
                         )
             else:
                 # if the target is not provided as xr.Dataarray we take the name of the target_data as new dimension name
+                _target_data_name_handling(target_data)
                 target_dim = target_data.name
                 target = xr.DataArray(
                     target, dims=[target_dim], coords={target_dim: target}
                 )
-            return target, target_dim, target_data
 
-        def _check_target_alignment(da, target_da):
-            # check alignment for all other axes, to avoid broadcasting enourmous arrays
-            # to do this on the axis level we will simply check if target_data has some coordinates
-            # that are not found in da (excluding all coordinates that belong to the current axis)
-            da_other_dims = set(da.dims) - set(self.coords.values())
-            target_da_other_dims = set(target_da.dims) - set(self.coords.values())
-            if not target_da_other_dims.issubset(da_other_dims):
-                raise ValueError(
-                    f"Found additional dimensions [{target_da_other_dims-da_other_dims}] in `target_data` not found in `da`. This could mean that the target array is not on the same position along other axes. Use grid.interp() to align the arrays."
-                )
+            _check_other_dims(target_data)
+            return target, target_dim, target_data
 
         _, dim = self._get_axis_coord(da)
         if method == "linear":
             target, target_dim, target_data = _parse_target(
                 target, target_dim, dim, target_data
             )
-            _check_target_alignment(da, target_data)
             out = linear_interpolation(
                 da,
                 target_data,
@@ -910,6 +942,7 @@ class Axis:
                 dim,  # in this case the dimension of phi and theta are the same
                 target_dim,
                 mask_edges=mask_edges,
+                bypass_checks=bypass_checks,
             )
         elif method == "conservative":
             # the conservative method requires `target_data` to be on the `outer` coordinate.
@@ -927,8 +960,6 @@ class Axis:
                 target, target_dim, target_data_dim, target_data
             )
 
-            _check_target_alignment(da, target_data)
-
             # check on which coordinate `target_data` is, and interpolate if needed
             if target_data_dim not in target_data.dims:
                 warnings.warn(
@@ -993,7 +1024,7 @@ class Axis:
 
     def _get_axis_coord(self, da):
         """Return the position and name of the axis coordiante in a DataArray."""
-        for position, coord_name in iteritems(self.coords):
+        for position, coord_name in self.coords.items():
             # TODO: should we have more careful checking of alignment here?
             if coord_name in da.dims:
                 return position, coord_name
@@ -1061,9 +1092,9 @@ class Grid:
             * None:  Do not apply any boundary conditions. Raise an error if
               boundary conditions are required for the operation.
             * 'fill':  Set values outside the array boundary to fill_value
-              (i.e. a Neumann boundary condition.)
+              (i.e. a Dirichlet boundary condition.)
             * 'extend': Set values outside the array to the nearest array
-              value. (i.e. a limited form of Dirichlet boundary condition.)
+              value. (i.e. a limited form of Neumann boundary condition.)
             * 'extrapolate': Set values by extrapolating linearly from the two
               points nearest to the edge
             Optionally a dict mapping axis name to seperate values for each axis
@@ -1334,7 +1365,7 @@ class Grid:
 
     def __repr__(self):
         summary = ["<xgcm.Grid>"]
-        for name, axis in iteritems(self.axes):
+        for name, axis in self.axes.items():
             is_periodic = "periodic" if axis._periodic else "not periodic"
             summary.append(
                 "%s Axis (%s, boundary=%r):" % (name, is_periodic, axis.boundary)
@@ -1364,9 +1395,9 @@ class Grid:
             * None:  Do not apply any boundary conditions. Raise an error if
               boundary conditions are required for the operation.
             * 'fill':  Set values outside the array boundary to fill_value
-              (i.e. a Neumann boundary condition.)
+              (i.e. a Dirichlet boundary condition.)
             * 'extend': Set values outside the array to the nearest array
-              value. (i.e. a limited form of Dirichlet boundary condition.)
+              value. (i.e. a limited form of Neumann boundary condition.)
 
             Optionally a dict with seperate values for each axis can be passed (see example)
         fill_value : {float, dict}, optional
@@ -1438,7 +1469,7 @@ class Grid:
         if a global 2D dataset should be interpolated on both X and Y axis, but it is
         only periodic in the X axis, we can do this:
 
-        >>> grid.interp(da, ['X', 'Y'], periodic={'X':True, 'Y':False})
+        >>> grid.interp(da, ["X", "Y"], periodic={"X": True, "Y": False})
         """
         return self._grid_func("interp", da, axis, **kwargs)
 
@@ -1466,7 +1497,7 @@ class Grid:
         if a global 2D dataset should be differenced on both X and Y axis, but the fill
         value at the boundary should be differenc for each axis, we can do this:
 
-        >>> grid.diff(da, ['X', 'Y'], fill_value={'X':0, 'Y':100})
+        >>> grid.diff(da, ["X", "Y"], fill_value={"X": 0, "Y": 100})
         """
         return self._grid_func("diff", da, axis, **kwargs)
 
@@ -1495,7 +1526,7 @@ class Grid:
         in both X and Y axis, but the fill value at the boundary should be different
         for each axis, we can do this:
 
-        >>> grid.min(da, ['X', 'Y'], fill_value={'X':0, 'Y':100})
+        >>> grid.min(da, ["X", "Y"], fill_value={"X": 0, "Y": 100})
         """
         return self._grid_func("min", da, axis, **kwargs)
 
@@ -1524,7 +1555,7 @@ class Grid:
         in both X and Y axis, but the fill value at the boundary should be different
         for each axis, we can do this:
 
-        >>> grid.max(da, ['X', 'Y'], fill_value={'X':0, 'Y':100})
+        >>> grid.max(da, ["X", "Y"], fill_value={"X": 0, "Y": 100})
         """
         return self._grid_func("max", da, axis, **kwargs)
 
@@ -1554,7 +1585,7 @@ class Grid:
         in both X and Y axis, but the fill value at the boundary should be different
         for each axis, we can do this:
 
-        >>> grid.max(da, ['X', 'Y'], fill_value={'X':0, 'Y':100})
+        >>> grid.max(da, ["X", "Y"], fill_value={"X": 0, "Y": 100})
         """
         return self._grid_func("cumsum", da, axis, **kwargs)
 
@@ -1712,24 +1743,20 @@ class Grid:
         axis : str, list of str
             Name of the axis on which to act
         **kwargs: dict
-            Additional arguments passed to `xarray.DataArray.sum`
-
+            Additional arguments passed to `xarray.DataArray.weighted.mean`
 
         Returns
         -------
         da_i : xarray.DataArray
             The averaged data
         """
-
         weight = self.get_metric(da, axis)
-        weighted = da * weight
-        # TODO: We should integrate xarray.weighted once available.
+        weighted = da.weighted(weight)
 
         # get dimension(s) corresponding
         # to `da` and `axis` input
         dim = self._get_dims_from_axis(da, axis)
-        # do we need to pass kwargs?
-        return weighted.sum(dim, **kwargs) / weight.sum(dim, **kwargs)
+        return weighted.mean(dim, **kwargs)
 
     def transform(self, da, axis, target, **kwargs):
         """Convert an array of data to new 1D-coordinates along `axis`.
@@ -1793,6 +1820,11 @@ class Grid:
         mask_edges : bool, optional
             If activated, `target` values outside the range of `target_data`
             are masked with nan, by default True. Only applies to 'linear' method.
+        bypass_checks : bool, optional
+            Only applies for `method='linear'`.
+            Option to bypass logic to flip data if monotonically decreasing along the axis.
+            This will improve performance if True, but the user needs to ensure that values
+            are increasing alon the axis.
         suffix : str, optional
             Customizable suffix to the name of the output array. This will
             be added to the original name of `da`. Defaults to `_transformed`.
