@@ -1,21 +1,18 @@
-from __future__ import print_function
-from future.utils import iteritems
+import numpy as np
 import pytest
 import xarray as xr
-import numpy as np
-from dask.array import from_array
 
-from xgcm.grid import Grid, Axis
+from xgcm.grid import Axis, Grid
 
-from .datasets import (
-    all_datasets,
-    nonperiodic_1d,
-    periodic_1d,
-    periodic_2d,
-    nonperiodic_2d,
+from .datasets import (  # noqa: F401
     all_2d,
+    all_datasets,
     datasets,
     datasets_grid_metric,
+    nonperiodic_1d,
+    nonperiodic_2d,
+    periodic_1d,
+    periodic_2d,
 )
 
 
@@ -327,8 +324,8 @@ def test_axis_cumsum(nonperiodic_1d, boundary):
         fill_value = 0.0 if boundary == "fill" else cumsum_c_raw[0]
         np.testing.assert_allclose(cumsum_c.data, np.hstack([fill_value, cumsum_c_raw]))
 
-    ## not much point doing this...we don't have the right test datasets
-    ## to really test the errors
+    # not much point doing this...we don't have the right test datasets
+    # to really test the errors
     # other_positions = {'left', 'right', 'inner', 'outer'}.difference({to})
     # for pos in other_positions:
     #     with pytest.raises(KeyError):
@@ -347,7 +344,7 @@ def test_axis_cumsum(nonperiodic_1d, boundary):
 def test_axis_neighbor_pairs_2d(
     periodic_2d, varname, axis_name, to, roll, roll_axis, swap_order
 ):
-    ds, periodic, expected = periodic_2d
+    ds, _, _ = periodic_2d
 
     axis = Axis(ds, axis_name)
 
@@ -450,7 +447,7 @@ def test_axis_diff_and_interp_nonperiodic_1d(nonperiodic_1d, boundary, from_cent
 def test_axis_diff_and_interp_nonperiodic_2d(
     all_2d, boundary, axis_name, varname, this, to
 ):
-    ds, periodic, expected = all_2d
+    ds, periodic, _ = all_2d
 
     try:
         ax_periodic = axis_name in periodic
@@ -476,7 +473,6 @@ def test_axis_diff_and_interp_nonperiodic_2d(
     if this == "center":
         if ax_periodic:
             data_left = np.roll(data, 1, axis=axis_num)
-            data_right = data
         else:
             pad_width = [
                 pad_left if i == axis_num else pad_none for i in range(data.ndim)
@@ -488,7 +484,7 @@ def test_axis_diff_and_interp_nonperiodic_2d(
                 ]
             )
             data_left = np.pad(data, pad_width, numpy_pad_arg[boundary])[the_slice]
-            data_right = data
+        data_right = data
     elif this == "left":
         if ax_periodic:
             data_left = data
@@ -543,27 +539,27 @@ def test_axis_errors():
     with pytest.raises(
         ValueError, match="Couldn't find a center coordinate for axis X"
     ):
-        x_axis = Axis(ds_noattr, "X", periodic=True)
+        _ = Axis(ds_noattr, "X", periodic=True)
 
     del ds_noattr.XG.attrs["axis"]
     with pytest.raises(ValueError, match="Couldn't find any coordinates for axis X"):
-        x_axis = Axis(ds_noattr, "X", periodic=True)
+        _ = Axis(ds_noattr, "X", periodic=True)
 
     ds_chopped = ds.copy().isel(XG=slice(None, 3))
     del ds_chopped["data_g"]
     with pytest.raises(ValueError, match="coordinate XG has incompatible length"):
-        x_axis = Axis(ds_chopped, "X", periodic=True)
+        _ = Axis(ds_chopped, "X", periodic=True)
 
     ds_chopped.XG.attrs["c_grid_axis_shift"] = -0.5
     with pytest.raises(ValueError, match="coordinate XG has incompatible length"):
-        x_axis = Axis(ds_chopped, "X", periodic=True)
+        _ = Axis(ds_chopped, "X", periodic=True)
 
     del ds_chopped.XG.attrs["c_grid_axis_shift"]
     with pytest.raises(
         ValueError,
         match="Found two coordinates without `c_grid_axis_shift` attribute for axis X",
     ):
-        x_axis = Axis(ds_chopped, "X", periodic=True)
+        _ = Axis(ds_chopped, "X", periodic=True)
 
     ax = Axis(ds, "X", periodic=True)
 
@@ -631,8 +627,6 @@ def test_create_grid_no_comodo(all_datasets):
 def test_grid_no_coords(periodic_1d):
 
     ds, periodic, expected = periodic_1d
-    grid_expected = Grid(ds, periodic=periodic)
-
     ds_nocoords = ds.drop_dims(list(ds.dims.keys()))
 
     coords = expected["axes"]
@@ -645,7 +639,7 @@ def test_grid_no_coords(periodic_1d):
 
 
 def test_grid_repr(all_datasets):
-    ds, periodic, expected = all_datasets
+    ds, periodic, _ = all_datasets
     grid = Grid(ds, periodic=periodic)
     r = repr(grid).split("\n")
     assert r[0] == "<xgcm.Grid>"
@@ -655,7 +649,7 @@ def test_grid_ops(all_datasets):
     """
     Check that we get the same answer using Axis or Grid objects
     """
-    ds, periodic, expected = all_datasets
+    ds, periodic, _ = all_datasets
     grid = Grid(ds, periodic=periodic)
 
     for axis_name in grid.axes.keys():
